;-------						
DrawLocation
;clear old enemies data
	CALL ClearEnemyData
	CALL ClearItemsData					;clear data of items and switches in curent location

;----------
;find location data address and put in into HL
	LD HL,(LevelMap)				
	LD DE,12							;each location address is 2 bytes
	SUB HL,DE							;location (1,1) is (LevelMap-12)+1*2+10*1
										;Location (3,2) is (LevelMap-12)+3*2+10*2
										;10=5*2 because map row has 5 locations
	LD A,(LocationX)
FindLocation1
	INC HL								;adjust address for proper collumn
	INC HL
	DEC A
	JR NZ, FindLocation1	
	
	LD A,(LocationY)
FindLocation2
	INC HL								;adjust address for proper row
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	INC HL
	DEC A
	JR NZ, FindLocation2
	
	LD E,(HL)
	INC HL
	LD D,(HL)
	EX DE,HL							;HL now contains room data start				
	

;-----------
DrawLocation2		
	LD (CurrentLocation),HL				;store start of current room data to proper cell
		
	LD A,64
	LD (WhichSegment),A			
		
;drawing tile blocks		
LocationBlocks1
		LD A, (HL)						;number of tile loaded in A
		OR A
		JR Z, LocationBlocks2			;if (HL)=0 that means either end of drawing blocks
										;or going to next of three screen blocks
										; (increasing H by 8)
		
		CALL CountTileAddress			;counts address of tile graphics and put it in DE
		
		INC HL 
		LD A,(HL)
		LD B,A
		INC HL
		LD A,(HL)
		LD C,A							;BC is loaded with block size
				
		INC HL
										;HL containing  room data address stored
		LD A,(HL)			
		
		PUSH HL		
		LD L,A							;HL is loaded with block
		LD A, (WhichSegment)			;left upper corner
		LD H,A	
	
		CALL TileBlock					;call block drawing routine drawing a block	of tiles
		
		POP HL							;restore room block data address in HL
		INC HL							;start of next block
		JR LocationBlocks1
		
;go to next screen part / end drawing blocks		
LocationBlocks2
		INC HL							;go to next byte of room data
		LD A,(WhichSegment)
		CP 80							;checks if last screen part
		JR Z, LocationTiles1 			;if so end drawing blocks
		ADD A,8							;if not go to another screen part
		LD (WhichSegment),A
		JR LocationBlocks1

;----------------		
;draw single tiles
LocationTiles1
		LD A,64
		LD (WhichSegment),A
		
LocationTiles2
		LD A, (HL)						;number of tile	loaded IN A
		
		OR A
		JR Z, LocationTiles3			;if (HL)=0 that means either end of drawing tiles
										;or going to next of three screen segments
										;(increasing H by 8)
		
		CALL CountTileAddress			;count address of tile graphics and put it in DE

		INC HL
		PUSH HL							;store room data address
		LD A, (HL)
		LD L,A
		LD A, (WhichSegment)
		LD H,A							;put upper left tile corner into HL
					
		CALL DrawTile					;draw single tile
		
		POP HL							;restore room data address
		INC HL							;go to next room data byte
		JR LocationTiles2
					
LocationTiles3
		INC HL							;go to next byte of room data
		LD A,(WhichSegment)
		CP 80							;checks if last screen part
		JR Z, LocationEnemies			;if so end drawing blocks
		ADD A,8							;if not go to another screen part
		LD (WhichSegment),A
					
		JR LocationTiles2

;--------------		
LocationEnemies
		LD A,(HL)						;are any enemies present in current location?
		LD (EnemyStatus),A
		OR A
		JR Z,LocationObjects			;no
		
		CP 1							;fan?
		JR NZ,LocationEnemies2			
		;read fan center data
		INC HL
		LD D,(HL)
		INC HL
		LD E,(HL)
		INC HL
		
		PUSH HL
		CALL SetFan						;prepare fan data
		POP HL
		JR LocationObjects
		
LocationEnemies2
		CALL SetEnemyShip	
		
		
LocationObjects
		CALL DrawItems
		CALL DrawSwitches
		CALL DrawDoors

		RET

WhichSegment	.BYTE 0	
;-----------------------
TileBlock
;HL position of left upper corner
;BC block size
;DE tile address IN memory		
		
Tilebl1	PUSH BC
		PUSH DE
		PUSH HL

;draws one horizontal line of tiles
Tilebl2		PUSH BC
			PUSH DE
			PUSH HL
			CALL DrawTile; draws A single tile

			POP HL
			POP DE ; restores screen position AND start address of data
	
			LD A,(DE)
			LD B,A ;reads tile width from memory AND writes to B
	
			LD A,L
			ADD A,B
			LD L,A; counts horizontal coordinate of next tile IN the line
	 			;(Adds B-tile size to coordinate of previous tile)
			
			POP BC	;writes block size to BC
					;decreases number of tiles IN the line yet to do
			DJNZ Tilebl2
		
		
; draws C lines of tiles
	POP HL
	POP DE
	
	INC DE
	LD A,(DE)	;reads tile height from memory
	LD C,A	;writes tile height to C	
	DEC DE	
	
		LD B,L;B is used to check carry		
		LD A,L	
Tilebl3	ADD A,32 ;go C lines down,because C=tile height
		DEC C
		JR NZ, Tilebl3
		LD L,A;changes vertical position to next collumn of tiles (moves it C chars down)
		
		;checks if new tile begins IN next of three screen blocks
		;carry appears when L<B(number exceeded 256)
		;A contains already L
		
		SUB B	
		JR NC, Tilebl4	;if no carry, just do nothing
		LD A,H
		ADD A,8
		LD H,A		;if carry change address properly
		
				
Tilebl4	POP BC; writes block size to BC
		DEC C	; decreases number of columns yet to do
		JR NZ, Tilebl1
		
		RET
		
			
;--------------------------------------
;This rouine draws a single colorful tile  on the screen

;HL contains address of the upper left corner of the tile		
;BC contains size of the tile (it is read by this rouine)
;DE contains start address of tile in memory	

;--------------------------------------
;draw a single tile  on the screen
	;HL: upper left tile corner size	
	;BC: tile size (not a parameter but read by this routine from tile data)
	;DE: tile memory address	

;FIRST BYTE OF 8 BYTES IN A CHAR CANNOT BE 16, BECAUSE PROGRAM WILL INTERPRET IT AS AN EMPTY SQUARE!!!
		
DrawTile
	DI
	LD A,(DE)				;load tile size to BC
	LD B,A 					;x-width
	INC DE
	LD A,(DE)
	LD C,A 					;y-height	
	INC DE			
				
	PUSH HL					;store position
	PUSH BC					;store size
			
	LD(StoreStack),SP		;SP=DE
	EX HL,DE				;stack points to tile graphics
	LD SP,HL
	EX HL,DE

	LD IXH,B
	LD IXL,C
	
DrawTile1	
	LD C,H				;store H for top of characters in the current row
	LD A,L				;store L
	
	LD B,IXH			;number of characters in the row

DrawTile2
	POP DE				;read 2 bytes of tile graphics
	EX AF,AF'			;work with alternate A
	LD A,E
	CP 16				;16 codes empty square
	JR NZ,DrawTile3		;normal, not empty character
		
	DEC SP				; POP DE increased SP by 2 and we skip only 1 byte, so decrement SP
	JR DrawTile4		;empty char, skip data drawing
		

DrawTile3
		
;draw 8 bytes, one character
		;POP DE was already done
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
		INC H
		
		POP DE				;draw two bytes
		LD (HL),E
		INC H
		LD (HL),D
	
DrawTile4
		EX AF,AF'			;back to normal A again		
		LD H,C				;restore H
		INC L				;go right to next character
		DJNZ DrawTile2		;repeat until whole row of characters is drawn
		
;restore leftmost character of the row and go one char down
		LD H,C				;restore H
		ADD A,32			;A stored L
		LD L,A				;now L=L+32
		JR NC,DrawTile5

;change H only if going to next screen segment	
			LD A,H
			ADD A,8
			LD H,A
					
DrawTile5	
		DEC IXL
		JR NZ,DrawTile1
		
		
		LD HL,0				;DE points to attributes now
		ADD HL,SP
		EX HL,DE
		
		LD SP,(StoreStack)	;restore old stack		
	
		
; graphics are ready, now paint attributes
; DE is already increased to next byte where attributes data begin
						
		POP BC
		POP HL		
							
;ATTRIBUTES
	;BC: tile size (B-width C-height)
	;DE:start of attributes data		
	;HL: screen address of left upper tile corner				
TileAttrs		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A ;transform graphics address to attribute address
			
TileAttr1
	 	PUSH BC
		PUSH HL
		
;draw one row of attributes
TileAttr2
	    LD A,(DE)
		OR A				;check if attribute is set to 0
		JR Z, TileAttr3		; if so, don't paint attribute and leave the square as it is
		LD (HL),A			; paint attribute
TileAttr3
		INC HL 				;go to next square in the row
		INC DE
		DJNZ TileAttr2
	
		POP HL
		PUSH DE  
		LD DE,32 
		ADD HL,DE			;go to next row of attributes
		POP DE 	 
		POP BC
		DEC C 				;decrease number of rows left to do 
		JR NZ, TileAttr1
		EI
		RET
		
		
;---------------------------
CountTileAddress	
	LD BC,TileAddrData-2
CountTileAddress2
	INC BC
	INC BC
	DEC A
	JR NZ,CountTileAddress2		
	;position of tile address in memory placed in BC	
	LD A,(BC)
	LD E,A
	INC BC
	LD A,(BC)
	LD D,A	;tile address placed IN DE
	RET
