;----------------------
ClearPlayer
;clear ship and satelites at old position

ClearShip
	LD HL,(ShipHL)
	LD DE,(ShipSprite)
	CALL XDrawShip

ClearSatelite1
;clear satelite 1 using old coordinates
	LD A,(Sat1OnScreen)			;is satelite in screen visible area?
	OR A
	JR Z,ClearSatelite2			;if it isn't don't do anything with it

	LD HL,(Satelite1HL)
	LD DE,(Satelite1Sprite)
	CALL XDrawSatelite			;XOR over graphics = clear graphics

;clear satelite2 using old coordinates
ClearSatelite2
	LD A,(Sat2OnScreen)			;is satelite in screen visible area?
	OR A
	RET Z						;if it isn't don't do anything with it

	LD HL,(Satelite2HL)
	LD DE,(Satelite2Sprite)
	CALL XDrawSatelite			;XOR over graphics = clear graphics
	
	RET	

;---------------------------
MovePlayer	
;read player input and change ship position appropriately
	LD A,(PlayerInput)
	
	OR A
	PUSH AF
	CALL Z,ReadKeys
	POP AF
	
	CP 1
	PUSH AF
	CALL Z,ReadKempston
	POP AF
	
	CP 2
	CALL Z,ReadSinclair
	
	
;now input (from keyboard or joystick is in E)	

MovePlayerRight	
	BIT 0,E
	JR Z,MovePlayerLeft	
	LD HL,ShipCenterX
	INC (HL)
		LD A,(HL)				;right edge reached?
		CP 216				
		JR NC,GoScreenRight
	JR MovePlayerDown
	
MovePlayerLeft
	BIT 1,E
	JR Z,MovePlayerDown
	LD HL,ShipCenterX
	DEC (HL)
		LD A,(HL)				;left edge reached?
		CP 10				
		JR C,GoScreenLeft
	
MovePlayerDown
	BIT 2,E
	JR Z,MovePlayerUp
	LD HL,ShipCenterY
	INC (HL)
		LD A,(HL)				;bottom edge reached?
		CP 184				
		JR NC,GoScreenDown
	JR ChangeRotation

MovePlayerUp
	BIT 3,E
	JR Z,ChangeRotation
	LD HL,ShipCenterY
	DEC (HL)
		LD A,(HL)				;top edge reached?
		CP 10				
		JR C,GoScreenUp
		
ChangeRotation
	BIT 4,E		
	RET Z
;fire pressed, change satelites rotation - left or right
	LD A,(RotationWaitTime)		;enough time elapsed since last change?
	OR A
	RET NZ						;no, still counting down to 0

	LD A,(Difficulty)
	OR A
	JR NZ,ChangeRotation2		;on easy difficulty don't count turns
	
	LD A,(RotationNr)
	OR A
	RET Z						;can't change rotation if no changes left

	DEC A						;decrease and display number of changes left
	LD (RotationNr),A
	LD HL,16384+157
	CALL DrawGameMenuNumbers2
	
ChangeRotation2	
	LD A,(Rotation)
	CPL
	LD (Rotation),A
	
	LD A,25
	LD (RotationWaitTime),A
	RET
;----------------------
;reaching playing area edge - changing screen
GoScreenLeft
	CALL ClearPlayingArea
	LD A,214
	LD (ShipCenterX),A				;change ship position
	LD HL,LocationX
	DEC (HL)						;change location - go left
	CALL ChangeLocation
	RET

GoScreenRight
	CALL ClearPlayingArea
	LD A,12							;change ship position
	LD (ShipCenterX),A
	LD HL,LocationX
	INC (HL)						;change location - go right
	CALL ChangeLocation
	RET

GoScreenUp
	CALL ClearPlayingArea
	LD A,182
	LD (ShipCenterY),A
	LD HL,LocationY
	DEC (HL)						;change location - go right
	CALL ChangeLocation
	RET


GoScreenDown
	CALL ClearPlayingArea
	LD A,12
	LD (ShipCenterY),A
	LD HL,LocationY
	INC (HL)						;change location - go right
	CALL ChangeLocation
	RET

;------		
ChangeLocation
	CALL DrawLocation
	CALL ClearEnemies
	RET	
	
;---------------------	
ReadKeys
;the aim is to set bits of A register as it would be kempston joystick input
	LD D,0
	LD BC,57342; O,P	
	IN A,(C)
	CPL
	AND 00000011B
	LD D,A
	LD BC,64510;Q
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	OR D		
	LD D,A		
	LD BC,65022;A
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	OR D
	LD D,A
	LD BC,32766
	IN A,(C)
	CPL
	AND 1
	RLA
	RLA
	RLA
	RLA
	OR D ;ready
	LD E,A
	RET

ReadKempston
	LD BC,31
	IN A,(C)
	LD E,A
	RET
	
ReadSinclair
	LD BC,0xeffe
	IN A,(C)
	CPL
	LD B,0
	
	BIT 3,A
	JR Z,ReadSinclair2
	SET 0,B

ReadSinclair2
	BIT 4,A
	JR Z,ReadSinclair3
	SET 1,B
	
ReadSinclair3
	BIT 1,A
	JR Z,ReadSinclair4
	SET 3,B

ReadSinclair4
	BIT 2,A
	JR Z,ReadSinclair5
	SET 2,B

ReadSinclair5
	BIT 0,A
	JR Z,ReadSinclair6
	SET 4,B

ReadSinclair6
	LD E,B
	RET
	

	
;-------------------------
DrawPlayer
;draw player ship and satelites at new position
	LD A,(ShipCenterX)
	SUB 8						;correct to find left top corner
	LD B,A
	LD A,(ShipCenterY)
	SUB 8
	LD C,A
	CALL FindShipSprite
	LD (ShipHL),HL
	LD (ShipSprite),DE
	CALL XDrawShip
	
	
;draw satelite1
DrawSatelite1
	LD A,(Sat1OnScreen)			;is satelite in screen visible area?
	OR A
	JR Z,DrawSatelite2			;if it isn't don't do anything with it
	
	LD A,(Satelite1CenterX)
	LD B,A
	LD A,(Satelite1CenterY)
	LD C,A
	CALL FindSateliteSprite		;after that routine HL= screen position, DE=graphics
	LD (Satelite1HL),HL			;store position
	LD (Satelite1Sprite),DE		;store sprite
	
	CALL XDrawSatelite			;draw satelite 1 using DE and HL

	
;draw satelite2
DrawSatelite2
	LD A,(Sat2OnScreen)			;is satelite in screen visible area?
	OR A
	RET Z						;if it isn't don't do anything with it

	LD A,(Satelite2CenterX)
	LD B,A
	LD A,(Satelite2CenterY)
	LD C,A
	CALL FindSateliteSprite		;after that routine HL= screen position, DE=graphics
	LD (Satelite2HL),HL			;store position
	LD (Satelite2Sprite),DE		;store sprite
	
	CALL XDrawSatelite			;draw satelite 1 using DE and HL
	RET
		
;---------------------------------------------------
FindSateliteSprite
;B: satelite x, C: satelite Y
	CALL XYtoHL					;put into HL screen coordinates of Satelite
	PUSH HL						

;A after subroutine contains shift
	LD HL,Satelite0					;start of satelite data
	LD DE,30 					;size of one satelite frame
	OR A
	JR Z,FindSateliteSprite3
	
FindSateliteSprite2
	ADD HL,DE
	DEC A
	JR NZ,FindSateliteSprite2
	
FindSateliteSprite3
	EX DE,HL					;start of sprite data in DE
	POP HL
	RET
	
;--------------------------	
XDrawSatelite
;draw (or clear satelite using XOR method)
	
	LD B,10;					;number of lines
	
XDrawSatelite2	
	LD A,(DE)					;Satelite is 3 bytes wide
	XOR (HL)
	LD (HL),A
	INC DE
	INC L
	
	LD A,(DE)
	XOR (HL)
	LD (HL),A
	INC DE
	INC L
	
	LD A,(DE)
	XOR (HL)
	LD (HL),A
	INC DE
	
	DEC L
	DEC L
	CALL OnePixelLineDown
	DJNZ XDrawSatelite2
	RET
	
;-----------------------------------
FindShipSprite
;draw (or clear Ship using XOR method)
;B: Ship X, C: Ship Y

	CALL XYtoHL					;put into HL screen coordinates of Ship
	PUSH HL						

;A after subroutine contains shift
	LD HL,Ship0					;start of Ship data
	LD DE,48 					;size of one Ship frame
	OR A
	JR Z,FindShipSprite3
	
FindShipSprite2
	ADD HL,DE
	DEC A
	JR NZ,FindShipSprite2
	
FindShipSprite3
	EX DE,HL					;start of sprite data in DE
	POP HL
	RET
;-----------------	
	
XDrawShip	
;drawing Ship sprite
	LD B,16;					;number of lines
	
XDrawShip2	
	LD A,(DE)					;Ship is 3 bytes wide
	XOR (HL)
	LD (HL),A
	INC DE
	INC L
	
	LD A,(DE)
	XOR (HL)
	LD (HL),A
	INC DE
	INC L
	
	LD A,(DE)
	XOR (HL)
	LD (HL),A
	INC DE
	
	DEC L
	DEC L
	CALL OnePixelLineDown
	DJNZ XDrawShip2
	RET

;---------------------------------------------
CheckCollisions
	XOR A
	LD (CollisionStatus_Ship),A	
	LD (CollisionStatus_Sat),A	
	
	
	
	LD HL,(Satelite1HL)				;satelite 1
	LD DE,(Satelite1Sprite)
	LD A,(Sat1OnScreen)				;if satelite is out of screen, don't test its collisions
	OR A
	CALL NZ,CheckCollissionSatelite	
	
	LD HL,(Satelite2HL)				;satelite 2
	LD DE,(Satelite2Sprite)
	LD A,(Sat2OnScreen)				
	OR A
	CALL NZ,CheckCollissionSatelite	
	
	CALL CheckCollisionShip			;ship	
	RET
	
	
;-------------------
CheckCollissionSatelite
	LD C,10							;number of horizontal lines=satelite size

CheckCollisionSatelite1	
	LD B,3							;width
	
CheckCollisionSatelite2
	LD A,(DE)						;load Satelite sprite data 
	OR (HL)							;and "draw" it virtually using OR method  over Satelite already drawn at screen
	CP (HL)							;did it change anything? (yes=collision with background)		
	JR Z,CheckCollisionSatelite3		;no collisiom
	
	LD A,1
	LD (CollisionStatus_Sat),A			;mark collision
	
CheckCollisionSatelite3	
	INC L							;next byte in the line
	INC DE							;next sprite data
	DJNZ CheckCollisionSatelite2
	
	DEC L							;go to first byte in the line
	DEC L
	DEC	L
	CALL OnePixelLineDown			;next line
	DEC C
	JR NZ,CheckCollisionSatelite1	
	RET

	
;-------------------
CheckCollisionShip
;we draw sprites using XOR method
;if we draw  a sprite over background (like wall) some pixel of player sprite will dissapear as 1 XOR 1=0
;if we draw the ship again over it using OR the picture will change as erased pictures will appear again
;but if ship had no contact with background (lit  pixels), drawing it again with OR won't change anything
;it is the rule used here to check ship collisions
	LD HL,(ShipHL)			
	LD DE,(ShipSprite)
	
	LD C,16							;number of horizontal lines=ship size

CheckCollisionShip1	
	LD B,3							;width
	
CheckCollisionShip2
	LD A,(DE)						;load ship sprite data 
	OR (HL)							;and "draw" it using OR method  over ship already drawn at screen
	CP (HL)							;did it change anything? (yes=collision with background)		
	JR Z,CheckCollisionShip3		;no collisiom
	
	LD A,1
	LD (CollisionStatus_Ship),A			;mark collision
	
CheckCollisionShip3	
	INC L							;next byte in the line
	INC DE							;next sprite data
	DJNZ CheckCollisionShip2
	
	DEC L							;go to first byte in the line
	DEC L
	DEC	L
	CALL OnePixelLineDown			;next line
	DEC C
	JR NZ,CheckCollisionShip1	
	
	RET
	
;--------------------
CollisionResult	
;collision satelite - lose energy
;colision ship - lose life (or pick up item, touch switch)
	LD A,(CollisionStatus_Sat)
	OR A
	CALL NZ,Collision_Sat			;decrease energy iof satelites ollide with background
	
	LD A,(CollisionStatus_Ship)		
	OR A
	RET Z							;no ship collision
	JR Collision_Ship
	
;--------	
;satelites	
Collision_Sat
	LD A,2
	OUT (254),A
	LD A,(Energy)					;decrease energy
	DEC A
	DEC A
	LD (Energy),A
	OR A
	JP Z,LifeLost
	
	CALL DrawEnergy
	LD A,16
	OUT (254),A
	RET	
;--------

Collision_Ship
;check picking up item
	LD A,(CurrentItemNr)
	OR A
	JR Z,Collision_Ship2			;no item in curent room
	
	CALL CheckTakingItem
	LD A,(TouchStatus)				;is ship touching the item
	OR A
	JR Z,Collision_Ship2			;no taking item
;taking item	
	CALL TakeItem
	RET

Collision_Ship2	
;check touching switch	
	LD A,(CurrentSwitchNr)
	OR A
	JR Z,Collision_Ship3			;no switch in this room
	
	CALL CheckTouchingSwitch
	LD A,(TouchStatus)				;is ship touching the switch
	OR A
	JR Z,Collision_Ship3			;no touching
	
	CP 1
	RET Z							;touching  switch which is off does nothing, neither crash nor action
	
	CALL PushSwitch					;push switch and open door
	RET

Collision_Ship3
	JP LifeLost	
	
	

;----------------------------
XYtoHL
;turn XY into HL screen coordinates
;B contains X, C contains Y 
 
	LD  A,C
	AND  00000111b
	LD  H,A
 	LD  A,C
 	RRA 
 	RRA 
 	RRA 
 	AND  00011000b
 	OR  H
 	OR  01000000b
 	LD  H,A					;H found
 
 	LD  A,B
 	RRA 
 	RRA 
 	RRA 
 	AND  00011111b
 	LD  L,A
 	LD  A,C
 	RLA 
 	RLA 
 	AND  11100000b
 	OR  L
 	LD  L,A					;L found
 	
 	LD A,B
 	AND 00000111b			;shift found
 	
 	RET 

;-------------
OnePixelLineDown
;requires HL as screen address
	INC H ;go to new line
	LD A,H
	AND 00000111B
	OR A ;h=0 -> going to new character line
	RET NZ; new line belongs to the same character line
	
	LD A,L ;going to new character line
	ADD A,32
	LD L,A
	RET C;going to new screen segment, if e.g h was 71, now it is 72
	
	LD A,H
	SUB 8
	LD H,A; decrease h if we are in the same screen segment
	RET


;--------------------
ClearScreen
;	LD A,1
	XOR A
	OUT (254),A 
	LD (23624),A		;border colour for ROM
	
	XOR A
	LD HL,16384
	LD (HL),A
	LD DE, 16385
	LD BC, 6144
	LDIR			
	
	LD A,71
	LD (HL),A
	LD BC,767
	LDIR

	RET

;---------------------
ClearPlayingArea

;clears playing area without touching the menu
	DI
	LD (StoreStack),SP
	
;graphics	
	LD A,192
	LD BC,0
	LD HL,16384-32+28
	LD DE,32

ClearPlayingArea2
	ADD HL,DE
	LD SP,HL
	
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	
	DEC A
	JR NZ,ClearPlayingArea2
	
;attributes
	LD A,24
	LD BC,71*256+71
	LD HL,22528-32+28
	LD DE,32

ClearPlayingArea3
	ADD HL,DE
	LD SP,HL
	
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	PUSH BC
	
	DEC A
	JR NZ,ClearPlayingArea3	
	
	LD SP,(StoreStack)
	EI
	RET

	
	